@@ -31,6 +31,7 @@
 //! }
 //! ```
 #![forbid(unsafe_code)]
+#![deny(missing_docs)]
 
 use git2::{Config, Cred, CredentialHelper, RemoteCallbacks};
 use semver::Version as SemverVersion;
@@ -52,17 +53,22 @@ pub use error::Error;
 
 static INDEX_GIT_URL: &str = "https://github.com/rust-lang/crates.io-index";
 
-/// A single version of a crate published to the index
+/// A single version of a crate (package) published to the index
 #[derive(Serialize, Deserialize, Clone, Debug)]
 pub struct Version {
     name: SmolStr,
     vers: SmolStr,
     deps: Arc<[Dependency]>,
     features: Arc<HashMap<String, Vec<String>>>,
+    /// It's wrapped in `Option<Box>` to reduce size of the struct when the field is unused (i.e. almost always)
+    /// https://rust-lang.github.io/rfcs/3143-cargo-weak-namespaced-features.html#index-changes
+    #[serde(default, skip_serializing_if = "Option::is_none")]
+    features2: Option<Box<HashMap<String, Vec<String>>>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     links: Option<Box<SmolStr>>,
     #[serde(with = "hex")]
     cksum: [u8; 32],
+    #[serde(default)]
     yanked: bool,
 }
 
@@ -93,11 +99,19 @@ impl Version {
         &self.cksum
     }
 
+    /// Explicit features this crate has. This list is not exhaustive,
+    /// because any optional dependency becomes a feature automatically.
+    ///
+    /// `default` is a special feature name for implicitly enabled features.
     #[inline]
     pub fn features(&self) -> &HashMap<String, Vec<String>> {
         &self.features
     }
 
+    /// Exclusivity flag. If this is a sys crate, it informs it
+    /// conflicts with any other crate with the same links string.
+    ///
+    /// It does not involve linker or libraries in any way.
     #[inline]
     pub fn links(&self) -> Option<&str> {
         self.links.as_ref().map(|s| s.as_str())
@@ -110,6 +124,7 @@ impl Version {
         self.yanked
     }
 
+    /// Where to find crate tarball
     pub fn download_url(&self, index: &IndexConfig) -> Option<String> {
         index.download_url(&self.name, &self.vers)
     }
@@ -120,11 +135,12 @@ impl Version {
 pub struct Dependency {
     name: SmolStr,
     req: SmolStr,
-    features: Box<[String]>,
+    /// Double indirection to remove size from this struct, since the features are rarely set
+    features: Box<Box<[String]>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     package: Option<Box<SmolStr>>,
-    #[serde(skip_serializing_if = "Option::is_none")]
-    kind: Option<DependencyKind>,
+    #[serde(default)]
+    kind: DependencyKind,
     #[serde(skip_serializing_if = "Option::is_none")]
     target: Option<Box<SmolStr>>,
     optional: bool,
@@ -132,41 +148,52 @@ pub struct Dependency {
 }
 
 impl Dependency {
+    /// Dependency's arbitrary nickname (it may be an alias). Use [`crate_name`] for actual crate name.
     #[inline]
     pub fn name(&self) -> &str {
         &self.name
     }
 
+    /// Semver version pattern
     #[inline]
     pub fn requirement(&self) -> &str {
         &self.req
     }
 
+    /// Features unconditionally enabled when using this dependency,
+    /// in addition to [`has_default_features`] and features enabled through
+    /// parent crate's feature list.
     #[inline]
     pub fn features(&self) -> &[String] {
         &self.features
     }
 
+    /// If it's optional, it implies a feature of its [`name`], and can be enabled through
+    /// the crate's features.
     #[inline]
     pub fn is_optional(&self) -> bool {
         self.optional
     }
 
+    /// If `true` (default), enable `default` feature of this dependency
     #[inline]
     pub fn has_default_features(&self) -> bool {
         self.default_features
     }
 
+    /// This dependency is only used when compiling for this `cfg` expression
     #[inline]
     pub fn target(&self) -> Option<&str> {
         self.target.as_ref().map(|s| s.as_str())
     }
 
+    /// Dev or not
     #[inline]
     pub fn kind(&self) -> DependencyKind {
-        self.kind.unwrap_or_default()
+        self.kind
     }
 
+    /// Set if dependency's crate name is different from the `name` (alias)
     #[inline]
     pub fn package(&self) -> Option<&str> {
         self.package.as_ref().map(|s| s.as_str())
@@ -194,11 +221,15 @@ impl Dependency {
     }
 }
 
+/// Section in which this dependency was defined
 #[derive(Debug, Copy, Clone, Serialize, Deserialize, Eq, PartialEq, Hash)]
 #[serde(rename_all = "lowercase")]
 pub enum DependencyKind {
+    /// Used at run time
     Normal,
+    /// Not fetched and not used, except for when used direclty in a workspace
     Dev,
+    /// Used at build time, not available at run time
     Build,
 }
 
@@ -236,13 +267,7 @@ fn fetch_opts<'cb>() -> git2::FetchOptions<'cb> {
     fetch_opts
 }
 
-fn crate_prefix(crate_name: &str, separator: char) -> Option<SmolStr> {
-    if !crate_name.is_ascii() {
-        return None;
-    }
-
-    let mut accumulator = SmolStr::new();
-
+fn crate_prefix(accumulator: &mut String, crate_name: &str, separator: char) -> Option<()> {
     match crate_name.len() {
         0 => return None,
         1 => accumulator.push('1'),
@@ -250,32 +275,28 @@ fn crate_prefix(crate_name: &str, separator: char) -> Option<SmolStr> {
         3 => {
             accumulator.push('3');
             accumulator.push(separator);
-            accumulator.push_str(&crate_name[0..1]);
+            accumulator.push_str(crate_name.get(0..1)?);
         }
         _ => {
-            accumulator.push_str(&crate_name[0..2]);
+            accumulator.push_str(crate_name.get(0..2)?);
             accumulator.push(separator);
-            accumulator.push_str(&crate_name[2..4]);
+            accumulator.push_str(crate_name.get(2..4)?);
         }
     };
-    Some(accumulator)
+    Some(())
 }
 
 fn crate_name_to_relative_path(crate_name: &str) -> Option<String> {
-    if !crate_name.is_ascii() {
-        return None;
-    }
-
     let name_lower = crate_name.to_ascii_lowercase();
     let mut rel_path = String::with_capacity(crate_name.len() + 6);
-    rel_path.push_str(&crate_prefix(&name_lower, std::path::MAIN_SEPARATOR)?);
+    crate_prefix(&mut rel_path, &name_lower, std::path::MAIN_SEPARATOR)?;
     rel_path.push(std::path::MAIN_SEPARATOR);
     rel_path.push_str(&name_lower);
 
     Some(rel_path)
 }
 
-/// A single crate that contains many published versions
+/// A single crate
 #[derive(Serialize, Deserialize, Clone, Debug)]
 pub struct Crate {
     versions: Box<[Version]>,
@@ -310,6 +331,14 @@ impl Crate {
             let mut version: Version = serde_json::from_slice(line)
                 .map_err(|e| io::Error::new(io::ErrorKind::Other, e))?;
 
+            if let Some(features2) = version.features2.take() {
+                if let Some(f1) = Arc::get_mut(&mut version.features) {
+                    for (key, mut val) in features2.into_iter() {
+                        f1.entry(key).or_insert_with(Vec::new).append(&mut val);
+                    }
+                }
+            }
+
             // Many versions have identical dependencies and features
             if let Some(has_deps) = deps_dedupe.get(&version.deps) {
                 version.deps = Arc::clone(has_deps);
@@ -416,27 +445,33 @@ impl Crate {
         })
     }
 
-    /// Published versions of this crate sorted chronologically by date published
+    /// Unconstrained All versions of this crate sorted chronologically by date originally published
+    ///
+    /// Warning: may be yanked or duplicate
     #[inline]
     pub fn versions(&self) -> &[Version] {
         &self.versions
     }
 
-    /// Oldest version.
+    /// Unconstrained Earliest version
     ///
-    /// Warning: may not be the lowest version number.
+    /// Warning: may not be the lowest version number and may be yanked or duplicate
     #[inline]
     pub fn earliest_version(&self) -> &Version {
         &self.versions[0]
     }
 
-    /// Most recently published version. Warning: may not be the highest version.
+    /// Unconstrained Latest version
+    ///
+    /// Warning: may not be the highest version and may be yanked or duplicate
     #[inline]
     pub fn latest_version(&self) -> &Version {
         &self.versions[self.versions.len() - 1]
     }
 
-    /// Returns the highest version as per semantic versioning specification, including unstable versions.
+    /// Returns the highest version as per semantic versioning specification
+    ///
+    /// Warning: may be unstable or yanked or duplicate
     pub fn highest_version(&self) -> &Version {
         self.versions
             .iter()
@@ -449,6 +484,8 @@ impl Crate {
 
     /// Returns the highest version as per semantic versioning specification,
     /// filtering out versions with pre-release identifiers.
+    ///
+    /// Warning: may be yanked or duplicate
     pub fn highest_stable_version(&self) -> Option<&Version> {
         self.versions
             .iter()
@@ -458,6 +495,7 @@ impl Crate {
             .map(|(v, _)| v)
     }
 
+    /// Crate's unique registry name. Case-sensitive, mostly.
     #[inline]
     pub fn name(&self) -> &str {
         self.versions[0].name()
@@ -468,7 +506,9 @@ impl Crate {
 /// https://doc.rust-lang.org/cargo/reference/registries.html#index-format
 #[derive(Clone, Debug, Deserialize)]
 pub struct IndexConfig {
+    /// Pattern for creating download URLs. Use [`download_url`] instead.
     pub dl: String,
+    /// Base URL for publishing, etc.
     pub api: Option<String>,
 }
 
@@ -491,7 +531,8 @@ impl IndexConfig {
             new.push_str("/download");
             return Some(new);
         } else {
-            let prefix = crate_prefix(name, '/')?;
+            let mut prefix = String::with_capacity(5);
+            crate_prefix(&mut prefix, name, '/')?;
             Some(
                 self.dl
                     .replace("{crate}", name)
@@ -505,10 +546,16 @@ impl IndexConfig {
 
 #[cfg(test)]
 mod test {
-    use super::Crate;
-    use super::Index;
+    use super::*;
     use tempdir::TempDir;
 
+    #[test]
+    fn sizes() {
+        assert!(std::mem::size_of::<Version>() <= 128);
+        assert!(std::mem::size_of::<Crate>() <= 16);
+        assert!(std::mem::size_of::<Dependency>() <= 80);
+    }
+
     #[test]
     fn semver() {
         let c = Crate::from_slice(r#"{"vers":"1.0.0", "name":"test", "deps":[], "features":{}, "cksum":"1234567890123456789012345678901234567890123456789012345678901234", "yanked":false}
@@ -589,4 +636,15 @@ mod test {
 
         assert!(found_gcc_crate);
     }
+
+    #[test]
+    fn features2() {
+        let c = Crate::from_slice(br#"{"vers":"1.0.0", "name":"test", "deps":[], "features":{"a":["one"], "b":["x"]},"features2":{"a":["two"], "c":["y"]}, "cksum":"1234567890123456789012345678901234567890123456789012345678901234"}"#).unwrap();
+        let f2 = c.latest_version().features();
+
+        assert_eq!(3, f2.len());
+        assert_eq!(["one", "two"], &f2["a"][..]);
+        assert_eq!(["x"], &f2["b"][..]);
+        assert_eq!(["y"], &f2["c"][..]);
+    }
 }
